@@ -178,6 +178,23 @@ int open_cterm(int flags);
  */
 const char *xgetprogname(void);
 
+/**
+ * Wrapper for strtoll() that forbids leading spaces.
+ *
+ * @param str
+ *         The string to parse.
+ * @param end
+ *         If non-NULL, will hold a pointer to the first invalid character.
+ *         If NULL, the entire string must be valid.
+ * @param base
+ *         The base for the conversion, or 0 to auto-detect.
+ * @param value
+ *         Will hold the parsed integer value, on success.
+ * @return
+ *         0 on success, -1 on failure.
+ */
+int xstrtoll(const char *str, char **end, int base, long long *value);
+
 /**
  * Process a yes/no prompt.
  *
@@ -359,7 +376,7 @@ size_t xread(int fd, void *buf, size_t nbytes);
  * writes.
  *
  * @return
-           The number of bytes written.  A value != nbytes indicates an error.
+ *         The number of bytes written.  A value != nbytes indicates an error.
  */
 size_t xwrite(int fd, const void *buf, size_t nbytes);
 
@@ -430,6 +447,19 @@ int xstrtofflags(const char **str, unsigned long long *set, unsigned long long *
  */
 long xsysconf(int name);
 
+/**
+ * Check for a POSIX option[1] at runtime.
+ *
+ * [1]: https://pubs.opengroup.org/onlinepubs/9799919799/basedefs/V1_chap02.html#tag_02_01_06
+ *
+ * @param name
+ *         The symbolic name of the POSIX option (e.g. SPAWN).
+ * @return
+ *         The value of the option, either -1 or a date like 202405.
+ */
+#define sysoption(name) \
+	(_POSIX_##name == 0 ? xsysconf(_SC_##name) : _POSIX_##name)
+
 #include <wchar.h>
 
 /**
