@@ -16,6 +16,7 @@
 #include "trie.h"
 #include <stddef.h>
 #include <sys/resource.h>
+#include <sys/types.h>
 #include <time.h>
 
 struct CFILE;
@@ -67,11 +68,18 @@ struct bfs_ctx {
 	bool status;
 	/** Whether to only return unique files (-unique). */
 	bool unique;
-	/** Whether to print warnings (-warn/-nowarn). */
-	bool warn;
 	/** Whether to only handle paths with xargs-safe characters (-X). */
 	bool xargs_safe;
 
+	/** Whether bfs was run interactively. */
+	bool interactive;
+	/** Whether to print warnings (-warn/-nowarn). */
+	bool warn;
+	/** Whether to report errors (-noerror). */
+	bool ignore_errors;
+	/** Whether any dangerous actions (-delete/-exec) are present. */
+	bool dangerous;
+
 	/** Color data. */
 	struct colors *colors;
 	/** The error that occurred parsing the color table, if any. */
@@ -98,6 +106,9 @@ struct bfs_ctx {
 	/** The number of files owned by the context. */
 	int nfiles;
 
+	/** The current file creation mask. */
+	mode_t umask;
+
 	/** The initial RLIMIT_NOFILE limits. */
 	struct rlimit orig_nofile;
 	/** The current RLIMIT_NOFILE limits. */
