@@ -22,7 +22,7 @@
 #include <unistd.h>
 
 /** Print some debugging info. */
-attr(printf(2, 3))
+_printf(2, 3)
 static void bfs_exec_debug(const struct bfs_exec *execbuf, const char *format, ...) {
 	const struct bfs_ctx *ctx = execbuf->ctx;
 
@@ -234,7 +234,7 @@ static char *bfs_exec_format_arg(char *arg, const char *path) {
 
 	char *last = arg;
 	do {
-		if (dstrncat(&ret, last, match - last) != 0) {
+		if (dstrxcat(&ret, last, match - last) != 0) {
 			goto err;
 		}
 		if (dstrcat(&ret, path) != 0) {
