@@ -206,6 +206,23 @@ static int xgetpart(const char **str, size_t n, int *result) {
 }
 
 int xgetdate(const char *str, struct timespec *result) {
+	// Handle @epochseconds
+	if (str[0] == '@') {
+		long long value;
+		if (xstrtoll(str + 1, NULL, 10, &value) != 0) {
+			goto error;
+		}
+
+		time_t time = (time_t)value;
+		if ((long long)time != value) {
+			errno = ERANGE;
+			goto error;
+		}
+
+		result->tv_sec = time;
+		goto done;
+	}
+
 	struct tm tm = {
 		.tm_isdst = -1,
 	};
@@ -324,6 +341,7 @@ end:
 		}
 	}
 
+done:
 	result->tv_nsec = 0;
 	return 0;
 
@@ -332,17 +350,3 @@ invalid:
 error:
 	return -1;
 }
-
-int xgettime(struct timespec *result) {
-#if _POSIX_TIMERS > 0
-	return clock_gettime(CLOCK_REALTIME, result);
-#else
-	struct timeval tv;
-	int ret = gettimeofday(&tv, NULL);
-	if (ret == 0) {
-		result->tv_sec = tv.tv_sec;
-		result->tv_nsec = tv.tv_usec * 1000L;
-	}
-	return ret;
-#endif
-}
